#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

import sys
import traceback
from typing import Dict, Optional

import html2text as html2textlib
import requests
from bs4 import BeautifulSoup


def html2text(html, withMd=True):
    h = html2textlib.HTML2Text(baseurl="", bodywidth=65535)
    if not withMd:
        h.ignore_emphasis = True
        h.ignore_images = True
        h.ignore_links = True
        h.ignore_tables = True
    return h.handle(html)


class UrlFetchRule:
    def __init__(self, lang: str, url: str, css: str, plain: bool, timeout: int = 15) -> None:
        self.lang: str = lang
        self.url: str = url
        self.css: str = css
        self.plain: bool = plain
        self.timeout: int = timeout

    def _fetch(self) -> Optional[str]:
        resp = requests.get(self.url, timeout=(self.timeout, self.timeout))
        resp.raise_for_status()
        resp_text = resp.text
        if self.css == '':
            return resp_text
        resp_bs = BeautifulSoup(resp_text, 'html5lib')
        sel_bs = resp_bs.select_one(self.css)
        if sel_bs is None:
            raise Exception('Markup selection did not match')
        if not self.plain:
            return sel_bs.prettify()
        return html2text(str(sel_bs), False)

    def fetch(self) -> Optional[str]:
        try:
            return self._fetch()
        except Exception as e:
            traceback.print_exc(file=sys.stderr)
            print('---- Exception safely handled', file=sys.stderr)
        return None


def main_decomposed_args(url_en: str,
                         url_es: str,
                         url_pt: str,
                         css_en: str = 'html',
                         css_es: str = 'html',
                         css_pt: str = 'html',
                         one_fails_all: bool = True,
                         plain: bool = True,
                         timeout=15,
                         ) -> Dict[str, Optional[str]]:
    sources = [
        UrlFetchRule('en', url_en, css_en, plain, timeout),
        UrlFetchRule('es', url_es, css_es, plain, timeout),
        UrlFetchRule('pt', url_pt, css_pt, plain, timeout),
    ]
    output: Dict[str, Optional[str]] = dict()
    for source in sources:
        tres = source.fetch()
        if tres is None and one_fails_all:
            raise Exception('Error with "One Fails All" enabled')
        output[source.lang] = tres
    if all(map(lambda a: a is None, output.values())):
        raise Exception('All requests failed')
    return output
